// RW Bumpmaster Project
// =====================
// Imageboard downloader + archiver demo project;  SAVE.EVERYTHING.ANON. :^)

// Filename: ./main_bumpmaster.cpp
// -The program's entry point

#include "incls_all.hpp"

void diag_bumpmaster(Bumpmaster const& bump);

//------------------------------------------------------------------------------
/**
 */
int main() try {
  puts("\n SAVE.EVERYTHING.ANON. :^)");

  Bumpmaster bump{"bumpmaster"};
  diag_bumpmaster(bump);

  puts("\n\n     --=^=--\n\n");

  // TEMP: TEST: init Bumpmaster functions; print Page 1 threads, 5 newest posts

  vector<string> const catalog_uris{
      // TEST: various board URI specification forms:

      // "https://alogs.space/robowaifu/catalog.json",
      // "https://alogs.space/robowaifu/index.html",
      "https://alogs.space/robowaifu/",
      // "alogs.space/robowaifu/",
      // "alogs.space robowaifu",
      // "htMp:///foo_bar..baz",

  };

  auto const thrd_lim{10};  // just the Page 1 threads
  auto const post_lim{5};   // just the 5 newest posts

  for (auto const& catalog_uri : catalog_uris) {  //

    // NOTE: this is intentional, not redundant work. a wide variety of possible
    // URI input patterns potentially need to be parsed up front--only one form
    // of which is the specifically correct one for a catalog's JSON dolldrop.
    auto const& [site, board] = bump.parse_cat_uri(catalog_uri);
    auto const cat_uri        = bump.bld_cat_uri(site, board);

    // pull the catalog dolldrop
    auto const cat_json = bump.pull_json_drop(cat_uri);

    // assemble all the thread's JSON URIs, in bump-order
    auto const bump_thrd_uris = bump.bump_thrd_uris(cat_json, site, board);

    for (auto t_idx{0}; t_idx < thrd_lim; ++t_idx) {         // pull P1 threads
      auto const thrd_uri  = bump_thrd_uris[t_idx];          //
      auto const thrd_json = bump.pull_json_drop(thrd_uri);  //

      // DESIGN: TODO: improve this looping/indexing algorithm

      auto const thrd_posts = bump.thrd_posts(thrd_json);  // read in all posts

      auto const sz = static_cast<int>(thrd_posts.size());      // cycle 5 posts
      if (sz > 0) {                                             //
        for (auto p_idx{sz - post_lim}; p_idx < sz; ++p_idx) {  //

          if (p_idx >= 0) {
            auto const p = thrd_posts[p_idx];

            cout << '\n'
                 << setw(33) << setfill(' ') << "r " << (p_idx + 1)
                 << "\n  ------------------------------------\n"
                 << setw(8) << setfill(' ') << p.id << "  :  " << p.date
                 << "\n\t' " << p.message << " '\n";
          }
        }

      } else
        puts("\n  no replies for this thread\n");

      puts("\n\n     --=^=--\n\n");
    }
  }

  // if (use_stamps)
  //   analyze_callgraph();

  // puts("\n\n     --=^=--\n\n");

  puts(" ...leaving main()\n");

} catch (runtime_error const& e) {
  cerr << "\n  runtime_error '" << e.what() << "' caught in main()\n\n";

} catch (exception const& e) {
  cerr << "\n  exception '" << e.what() << "' caught in main()\n\n";

} catch (...) {
  cerr << "\n  unknown error caught in main()\n\n";
}

//------------------------------------------------------------------------------
/**
 * @param bump
 */
void diag_bumpmaster(Bumpmaster const& bump) {  //

  prt_tag(bump.t(), "bump");
  prt_tag(bump.board().t(), "bump.board()");
  prt_tag(bump.catalog().t(), "bump.catalog()");
  prt_tag(bump.page().t(), "bump.page()");
  prt_tag(bump.post().t(), "bump.post()");
  prt_tag(bump.site().t(), "bump.site()");
  prt_tag(bump.thread().t(), "bump.thread()");
  prt_tag(bump.waifusearch().t(), "bump.waifusearch()");
  //
  prt_tag(bump.dolldrop().t(), "bump.dolldrop()");
}

//    --=^=--

// "Everyone's talking about IT & the Internet, and how advanced it's getting."

// rw_bumpmaster_demo.v220219
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
