// RW Bumpmaster
// ==============
// Imageboard Retrieval & Archiving Library
// -Part of the RW Foundations suite of libraries

// Filename: ./src/curator/bumpmaster/bumpmaster.cpp
// -Definitions of the baseline RW Bumpmaster sub-library

#pragma once

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
void from_json(Json const& j, Postdata& p) {  //

  if (! j["creation"].is_null())  // TEST:
    p.date = j["creation"];
  else {
    puts("  IS_null");
    p.date = "";
  }

  if (! j["message"].is_null())  // TEST:
    p.message = j["message"];
  else {
    puts("  IS_null");
    p.message = "";
  }

  if (! j["postId"].is_null())  // TEST:
    p.id = j["postId"];
  else {
    puts("  IS_null");
    p.id = 0;
  }
}

//------------------------------------------------------------------------------
void to_json(Json& j, Postdata const& p) {
  j = Json{{"creation", p.date}, {"message", p.message}, {"postId", p.id}};
}

//------------------------------------------------------------------------------
Bumpmaster_b::Bumpmaster_b() : Bumpmaster_b{"bumpmaster_b_obj"} {}

//------------------------------------------------------------------------------
Bumpmaster_b::Bumpmaster_b(std::string const& handle)
    : RW{handle, Type::Bumpmaster} {
  stamp("Bumpmaster_b()");
}

//------------------------------------------------------------------------------
auto Bumpmaster_b::parse_cat_uri(std::string const& cat_uri)
    -> std::tuple<std::string, std::string> {
  stamp("parse_cat_uri()");

  string site{""}, board{""};

  regex const ex{"(http|https)://([^/ ]*)/([^ #?]*)/([^ #?]*)"};
  std::cmatch match;

  if (regex_match(cat_uri.c_str(), match, ex)) {
    site  = string{match[2].first, match[2].second};
    board = string{match[3].first, match[3].second};
  }

  return make_tuple(site, board);
}

//------------------------------------------------------------------------------
// void Bumpmaster_b::chk_board_dir(std::string const& site_nm,
//                                  std::string const& board_nm) {
//   //
//   auto foo = site_nm;   // TEMP:
//   auto bar = board_nm;  //  "
// }

//------------------------------------------------------------------------------
auto Bumpmaster_b::bld_cat_uri(std::string const& host, std::string const& path)
    -> std::string {
  string const scheme{"https://"};
  string const page_nm{"catalog"};
  string const type{"json"};

  return {scheme + host + '/' + path + '/' + page_nm + '.' + type};
}

//------------------------------------------------------------------------------
auto Bumpmaster_b::bld_thrd_uri(std::string const& host,
                                std::string const& path,
                                unsigned const     thrd_id) -> std::string {
  string const scheme{"https://"};
  string const res{"res"};
  string const type{"json"};

  // DESIGN: for now we'll just adapt to Lynxchan's formats for various things
  // (like thread URIs), since our own site's servers use it. later, we'll
  // expand out to every mainline IB type using an approach mostly similar to
  // the one previously devised for BUMP software.

  return {scheme + host + '/' + path + '/' + res + '/' + to_string(thrd_id) +
          '.' + type};
}

//------------------------------------------------------------------------------
auto Bumpmaster_b::rd_thread_id(Json const& thread) -> unsigned {
  unsigned res{0};

  // cycle through the thread's fields
  for (auto const& [field_nm, field_dat] : thread.items()) {  //

    if (field_nm == "threadId") {
      res = field_dat;
      break;
    }
  }

  return res;
}

//------------------------------------------------------------------------------
auto Bumpmaster_b::bump_thrd_uris(Json const&        cat_json,
                                  std::string const& site,
                                  std::string const& board)
    -> std::vector<std::string> {
  std::vector<std::string> res{};
  res.reserve(cat_json.size());

  // cycle through all the board catalog's threads
  for (auto const& [implct_idx, thread] : cat_json.items()) {
    auto const thrd_id  = rd_thread_id(thread);
    auto const thrd_uri = bld_thrd_uri(site, board, thrd_id);

    res.push_back(thrd_uri);
  }

  return res;
}

//------------------------------------------------------------------------------
auto Bumpmaster_b::thrd_posts(Json const& thrd_json) -> std::vector<Postdata> {
  std::vector<Postdata> res{};
  res.reserve(thrd_json.size());

  for (auto const& [field_nm, field_dat] : thrd_json.items()) {  //

    if (field_nm == "posts") {  //

      for (auto const& [post_idx, post_dat] : field_dat.items()) {
        try {
          res.emplace_back(post_dat);  // will ctor a Postdata struct in place

        } catch (Json::exception const& e) {  // TEST:
          puts("\n\nhave Json exception in thrd_posts()\n\n");
        }
      }

      break;
    }
  }

  return res;
}

//------------------------------------------------------------------------------
Bumpmaster::Bumpmaster() : Bumpmaster{"bumpmaster_obj"} {}

//------------------------------------------------------------------------------
Bumpmaster::Bumpmaster(std::string const& handle) : Bumpmaster_b{handle} {
  stamp("Bumpmaster()");

  // preflight();
}

//------------------------------------------------------------------------------
void Bumpmaster::preflight() {
  stamp("preflight()");

  // TBD...
}

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

//------------------------------------------------------------------------------
auto Bumpmaster::pull_json_drop(std::string const& json_uri) -> Json {  //

  auto json_strm = dolldrop_.pull(json_uri);
  if (! is_empty(json_strm)) {  //

    auto const json = fstream_2_json(json_strm);
    if (! json.empty()) {  //

      return json;

    } else
      cerr << "\n\n  empty JSON data - pull_json_drop()\n\n";

  } else
    cerr << "\n\n  empty file stream - pull_json_drop()\n\n";

  return Json{};
}

#endif  // __has_include(<curl_easy.h>)

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v220219
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
